/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.fukurou.util;

import java.util.List;
import java.util.ArrayList;

/**
 * EXCELファイルを、イベント方式に準拠して、テキストとして読み込み処理を行います。
 * ExcelReaderEventイベントは、openGion形式のファイル読み取りに準拠した方法をサポートします。
 * ※ openGion形式のEXCELファイルとは、#NAME 列に、カラム名があり、#で始まる
 *    レコードは、コメントとして判断し、読み飛ばす処理の事です。
 *
 * このイベントクラスは、サブクラスを作成し、ExcelReader_HSSF、ExcelReader_XSSF、
 * ExcelReader_SS の excelReader メソッドの引数に指定します。
 * HSSF,XSSF は、イベントモデルで処理します。SS は、DOMモデルですが、(.xls/.xlsx)の
 * 両方をサポートします。
 * 実際は、POIUtil#excelReader( String , ExcelReaderEvent ) を使用すれば、拡張子に応じて
 * 使用するクラスを選択します。
 *
 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
 * @og.group ファイル入力
 *
 * @version  4.0
 * @author   Kazuhiko Hasegawa
 * @since    JDK5.0,
 */
public class ExcelReaderEvent {
	private String	nowShtNm	= null;		// 現在のシート名
	private int		nowShtNo	= -1;		// 現在のシート番号
	private int		nowRowNo	= -1;		// 現在の行番号
	private boolean	isNowSkip	= false;	// スキップ中かどうか(true:スキップ中)
	private boolean	isNowName	= false;	// カラム名配列の収集中かどうか(true:収集中)

	private int		 clmSize	= -1 ;		// カラムサイズ
	private String[] names		= null ;	// カラム名配列
	private String[] vals		= null ;	// 値の文字列配列(１行分)
	private boolean	useVals		= false;	// 値配列に設定されたか？

	private List<Integer> colList = null ;	// カラム番号:name が null かゼロ文字列の場合は、飛ばします。
	private List<String>  nmsList = null ;	// カラム名:カラム番号に対応したカラム名

	/**
	 * シートの読み取り開始時にイベントが発生します。
	 *
	 * 新しいシートの読み取り開始毎に、１回呼ばれます。
	 * 戻り値が、true の場合は、そのシートの読み取りを継続します。
	 * false の場合は、そのシートの読み取りは行わず、次のシートまで
	 * イベントは発行されません。
	 * 初期実装では、内部変数に、シート名、シート番号をセットし、true を返します。
	 *
	 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
	 *
	 * @param   shtNm  シート名
	 * @param   shtNo  シート番号(0～)
	 * @return  true:シートの読み取り処理を継続します/false:このシートは読み取りません。
	 */
	public boolean startSheet( final String shtNm,final int shtNo ) {
		nowShtNm	= shtNm;
		nowShtNo	= shtNo;

		return true;
	}

	/**
	 * シートの読み取り終了時にイベントが発生します。
	 *
	 * #columnNames( String[] ) や、#values( String[] ,int ) などは、行の処理が完了した時点で
	 * イベントが呼ばれるため、一番最後のレコードの終了条件が判りません。
	 * そこで、このイベントを呼ぶことで、シートの終了(=最終行の終了)処理を行うことができます。
	 * 
	 * 引数のシート番号は、参考情報で、#startSheet( String,int ) で呼ばれたシート番号と
	 * 比較できるようにしています。
	 * 初期実装では、行終了処理を実行します。
	 *
	 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
	 *
	 * @param   shtNo  シート番号(0～)
	 */
	public void endSheet( final int shtNo ) {
		endRow( -1 );					// 行終了処理
	}

	/**
	 * 読み取り状態の時に、rowNo,colNo にあるセルの値を引数にイベントが発生します。
	 *
	 * 戻り値が、true の場合は、その行の読み取りを継続します。
	 * false の場合は、その行の読み取りは行わず、次の行まで
	 * イベントは発行されません。
	 *
	 * openGion での標準的な処理は、colNo==0 の時に、val の先頭が、# で
	 * 始まる場合は、その行はスキップします。
	 * ここでの return は、#isSkip( int ) と逆になりますので、ご注意ください。
	 * 初期実装では、#NAME処理、行スキップ、行終了処理等を実行します。
	 *
	 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
	 *
	 * @param   val     文字列値
	 * @param   rowNo   行番号(0～)
	 * @param   colNo   列番号(0～)
	 * @return  読み取りするかどうか(true:読み取りする/false:読み取りしない)
	 * @see		#isSkip( int )
	 */
	public boolean value( final String val,final int rowNo,final int colNo ) {
		// 値が存在する場合のみ、処理する。
		if( val!=null && val.length()>0 ) {
			// 行番号が異なった場合は、行の終了処理を実行する。
			if( nowRowNo != rowNo ) {
				endRow( rowNo );						// 行終了処理
			}

			// 先頭カラムのコメント判断と、#NAME列判断
			if( colNo==0 && val.charAt(0)=='#' ) {
				if( "#NAME".equalsIgnoreCase( val ) && clmSize < 0 ) {	// clmSize < 0 は、#NAME未設定という意味
					isNowName = true ;
					colList = new ArrayList<Integer>() ;	// 初期化
					nmsList = new ArrayList<String>() ;		// 初期化
				}
				isNowSkip = !isNowName;						// #NAME の場合は、false(SKIPしない)
			}
			else if( isNowName ) {							// #NAME処理中
				colList.add( Integer.valueOf( colNo ) );
				nmsList.add( val );
			}
			else if( clmSize >= 0 ) {						// #NAME が設定済みという意味
				int indx = colList.indexOf( Integer.valueOf( colNo ) );		// カラム番号の位置が 値配列の配列番号
				if( indx >= 0 && indx < clmSize ) {
					vals[indx] = val;
					useVals = true;
				}
			}
		}

		return !isNowSkip;
	}

	/**
	 * rowNo を元に、この行をスキップするかどうか判定のイベントが発生します。
	 *
	 * ※ この処理は、行が新しくなると、false が返ります。つまり、カラムの値を求める
	 *    イベントやループの直前に入れることで、同一カラム処理のループ処理を
	 *    短縮するのが狙いです。
	 *    引数から、行のループに入れてしまいそうですが、行列のループに入れてください。
	 *
	 * 戻り値が、true の場合は、その行の読み取りをスキップします。
	 * false の場合は、読み取り処理を継続します。
	 * スキップ中は、行に関するイベントは発行されません。
	 *
	 * 値(val)を求める前に、行情報を入手し、スキップ中の現在行と同じ場合に、スキップ(=true)と判定します。
	 * スキップ中かどうかは、#value( String,int,int ) で、判定します。
	 * 初期実装では、スキップ中 かつ 現在行と同じ場合に、true を返します。
	 *
	 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
	 *
	 * @param   rowNo   行番号(0～)
	 * @return  スキップするかどうか(true:スキップする/false:スキップしない)
	 * @see		#value( String,int,int )
	 */
	public boolean isSkip( final int rowNo ) {
		return isNowSkip && nowRowNo == rowNo ;
	}

	/**
	 * 行の終了時に実行されます。
	 *
	 * ここでは、rowNo がブレークするか、シート終了時に呼ぶことで、
	 * 一番最後の行の処理を行います。
	 *
	 * #NAME 処理中の場合(isNowName == true) は、カラム名配列を作成して、
	 * columnNames イベントを呼び出します。
	 * 値配列が設定されている場合(useVals == true) は、values イベントを
	 * 呼び出します。
	 * #NAME 処理が完了している場合は、値配列の初期化を行います。
	 * そのうえで、スキップの解除(isNowSkip = false)と行データ
	 * 未設定(useVals = false)に、初期化します。
	 *
	 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
	 *
	 * @param   rowNo   行番号(0～)
	 * @see		#columnNames( String[] )
	 * @see		#values( String[],int )
	 */
	private void endRow( final int rowNo ) {
		if( isNowName ) {								// #NAME 処理中の場合
			clmSize = colList.size();
			names   = nmsList.toArray( new String[clmSize] );
			columnNames( names.clone() );				// イベント
			isNowName = false;
			nmsList = null ;		// 不要
		}
		else if( useVals ) {							// 値配列が設定されている場合
			values( vals.clone() , nowRowNo );			// イベント発行(現在行)
		}

		if( clmSize >= 0 ) {
			vals = new String[clmSize];					// 値配列の初期化
		}

		isNowSkip = false;						// スキップの解除
		useVals	  = false;						// 行データ未設定にする。
		nowRowNo  = rowNo;						// 現在行の設定
	}

	/**
	 * カラム名配列がそろった段階で、イベントが発生します。
	 *
	 * openGion での標準的な処理は、colNo==0 の時に、val の先頭が、#NAME
	 * で始まるレコードを、名前配列として認識します。
	 * #value( String,int,int ) で、この #NAME だけは、継続処理されます。
	 * その上で、#NAME レコードが終了した時点で、カラム名配列が完成するので
	 * そこで初めて、このメソッドが呼ばれます。
	 * 初期実装は、されていません。
	 *
	 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
	 *
	 * @param   names  カラム名配列
	 * @see		#value( String,int,int )
	 */
	public void columnNames( final String[] names ) {
	}

	/**
	 * row にあるセルのオブジェクト値がそろった段階で、イベントが発生します。
	 *
	 * 初期実装は、されていません。
	 *
	 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
	 *
	 * @param   vals    文字列値の１行分の配列
	 * @param   rowNo   行番号(0～)
	 */
	public void values( final String[] vals,final int rowNo ) {
	}
}
