/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.fukurou.util;

import java.io.InputStream;
import java.io.IOException;
import java.util.List;							// 6.0.3.0 (2014/11/13) XSSFイベントモデル
import java.util.ArrayList;						// 6.0.3.0 (2014/11/13) XSSFイベントモデル

// 6.0.3.0 (2014/11/13) XSSFイベントモデル
import org.apache.poi.xssf.eventusermodel.XSSFReader;
import org.apache.poi.xssf.model.SharedStringsTable;
import org.apache.poi.xssf.usermodel.XSSFRichTextString;
import org.apache.poi.openxml4j.opc.OPCPackage;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException ;
import org.xml.sax.Attributes;
// import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * POI による、Excel(xlsx)の読み取りクラスです。
 *
 * xlsx形式のEXCELを、イベント方式でテキストデータを読み取ります。
 * このクラスでは、XSSF(.xlsx)形式のファイルを、ExcelReaderEvent を介したイベントで読み取ります。
 * ExcelReaderEventイベントは、openGion形式のファイル読み取りに準拠した方法をサポートします。
 * ※ openGion形式のEXCELファイルとは、#NAME 列に、カラム名があり、#で始まる
 *    レコードは、コメントとして判断し、読み飛ばす処理の事です。
 *
 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
 * @og.group ファイル入力
 *
 * @version  6.0
 * @author   Kazuhiko Hasegawa
 * @since    JDK7.0,
 */
public final class ExcelReader_XSSF {
	/** このプログラムのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "6.0.3.0 (2014/11/13)" ;

	private static final String CR = System.getProperty("line.separator");

	/** XSSF(.xlsx)ファイル解析用の、SAXParser {@value} */
	public static final String SAX_PARSER = "org.apache.xerces.parsers.SAXParser" ;

	/**
	 * 引数ファイル(Excel)を、XSSFイベントモデルを使用してテキスト化します。
	 *
	 * ExcelReaderEventは、EXCEL読み取り処理用の統一されたイベント処理クラスです。
	 * openGion特有のEXCEL処理方法(#NAME , 先頭行#コメントなど)を実装しています。
	 * これは、HSSFやXSSFの処理を、統一的なイベントモデルで扱うためです。
	 * SSモデルが良いのですが、巨大なXSSF(.xlsx)ファイルを解析すると、OutOfMemoryエラーが
	 * 発生する為、個々に処理する必要があります。
	 * あくまで、読み取り限定であれば、こちらのイベントモデルで十分です。
	 *
	 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
	 *
	 * @param	fname 入力ファイル名
	 * @param	event イベント処理するオブジェクト
	 */
	public static void excelReader( final String fname , final ExcelReaderEvent event ) {
		OPCPackage pkg = null;

		try {
			pkg = OPCPackage.open( fname );
			XSSFReader rd = new XSSFReader( pkg );

			String[] sheetNames = getSheetNames( rd );

			XMLReader parser = XMLReaderFactory.createXMLReader( SAX_PARSER );
			SharedStringsTable sst = rd.getSharedStringsTable();
			SheetHandler handler = new SheetHandler( sst,event );		// ContentHandler のサブクラス
			parser.setContentHandler(handler);

			InputStream sheet = null;

			// Iterator<InputStream> sheets = rd.getSheetsData();
			// while(sheets.hasNext()) {
			//     sheet = sheets.next();
			//     ・・・・・
			// }
			// 形式で、全シート対象に処理できますが、シート名が取り出せません。

			for( int i=0; i<sheetNames.length; i++ ) {
				event.startSheet( sheetNames[i] , i );		// イベント処理
//				handler.setSheetName( sheetNames[i] , i );
				try {
					sheet = rd.getSheet("rId"+(i+1));			// シートIDは、rId# で取得できる。#は、1～
					InputSource sheetSource = new InputSource(sheet);
					parser.parse(sheetSource);
				}
				finally {
					Closer.ioClose( sheet );
				}
				event.endSheet( i );						// イベント処理
			}
		}
		catch( Exception ex ) {
			String errMsg = ".xlsxのファイル解析に失敗しました。"
								+ " filename=" + fname + "\n"
								+ ex.getMessage() ;
			throw new RuntimeException( errMsg , ex );
		}
	//	finally {
	//		if( pkg != null ) {
	//			pkg.revert();
	//			Closer.ioClose( pkg );				// OPCPackage を close すると、書き戻しされる。
	//		}
	//	}
	}

	/**
	 * この内部クラスは、XSSFイベントモデルに基づいた、xlsxファイルを SAX処理します。
	 *
	 * この処理のオリジナルは、https://svn.apache.org/repos/asf/poi/trunk/src/examples/src/org/apache/poi/xssf/eventusermodel/examples/FromHowTo.java です。
	 *
	 * DefaultHandler を継承しており、xlsx の シート処理を行い、カラム番号と値を取得します。
	 * このクラス自体は、内部で使用されるため、ExcelReaderEvent を引数に設定することで、
	 * 外部から、EXCELのセル情報の取得が可能です。
	 *
	 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
	 *
	 * @see		org.xml.sax.helpers.DefaultHandler
	 */
	private static final class SheetHandler extends DefaultHandler {
		private final SharedStringsTable	sst  ;
		private final ExcelReaderEvent		event;

		private String  lastContents ;
		private boolean isSST;

//		private String shtNm = null ;
//		private int    shtNo = -1;

		private int		rowNo = -1;		// 現在の行番号
		private int		colNo = -1;		// 現在の列番号

		private boolean isRowSkip = false;	// シートの読み取りを行うかどうか

		/**
		 * コンストラクター
		 *
		 * SharedStringsTable は、テキストの値を持っているオブジェクトです。
		 * ここで指定する ExcelReaderEvent に対して、パーサー処理の結果がセットされます。
		 *
		 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
		 *
		 * @param	sst   SharedStringsTableオブジェクト
		 * @param	event イベント処理するオブジェクト
		 */
		private SheetHandler( final SharedStringsTable sst , final ExcelReaderEvent event ) {
			this.sst   = sst;
			this.event = event;
		}

		/**
		 * 要素の開始通知を受け取ります。
		 *
		 * インタフェース ContentHandler 内の startElement メソッドをオーバーライドしています。
		 * パーサは XML 文書内の各要素の前でこのメソッドを呼び出します。
		 * 各 startElement イベントには対応する endElement イベントがあります。
		 * これは、要素が空である場合も変わりません。対応する endElement イベントの前に、
		 * 要素のコンテンツ全部が順番に報告されます。
		 * ここでは、タグがレベル３以上の場合は、上位タグの内容として取り扱います。よって、
		 * タグに名前空間が定義されている場合、その属性は削除します。
		 *
		 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
		 *
		 * @param	namespace	名前空間 ＵＲＩ
		 * @param	localName	前置修飾子を含まないローカル名。名前空間処理が行われない場合は空文字列
		 * @param	qname		前置修飾子を持つ修飾名。修飾名を使用できない場合は空文字列
		 * @param	attributes	要素に付加された属性。属性が存在しない場合、空の Attributesオブジェクト
		 * @see		org.xml.sax.helpers.DefaultHandler#startElement(String , String , String , Attributes )
		 */
		@Override
		public void startElement( final String namespace, final String localName, final String qname, final Attributes attributes ) {
			if( "row".equals(qname) ) {			// row
				rowNo = Integer.parseInt( attributes.getValue("r") ) - 1;	// 0 から始まる
				isRowSkip = false;
			}
			else if( isRowSkip ) { return ; }
			else if( "c".equals(qname) ) {		// c => cell
				String kigo = attributes.getValue("r") ;					// Excelの行列記号(A1 など)
				int[] rowCol = POIUtil.kigo2rowCol( kigo );					// Excelの行列記号を、行番号と列番号に分解します。

		//		rowNo = rowCol[0];			// 行番号････
				colNo = rowCol[1];			// カラム番号

				// Figure out if the value is an index in the SST
				isSST = "s".equals( attributes.getValue("t") );				// cellType
			}
			lastContents = "";		// なんでもクリアしておかないと、関数文字列を拾ってしまう。
		}

		/**
		 * 要素の終了通知を受け取ります。
		 *
		 * インタフェース ContentHandler 内の endElement メソッドをオーバーライドしています。
		 * SAX パーサは、XML 文書内の各要素の終わりにこのメソッドを呼び出します。
		 * 各 endElement イベントには対応する startElement イベントがあります。
		 * これは、要素が空である場合も変わりません。
		 *
		 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
		 *
		 * @param	namespace	名前空間 URI
		 * @param	localName	前置修飾子を含まないローカル名。名前空間処理が行われない場合は空文字列
		 * @param	qname		前置修飾子を持つ XML 1.0 修飾名。修飾名を使用できない場合は空文字列
		 * @see org.xml.sax.helpers.DefaultHandler#endElement(String , String , String )
		 */
		@Override
		public void endElement( final String namespace, final String localName, final String qname ) {
			isRowSkip = event.isSkip( rowNo );							// イベント

			if( isRowSkip ) { return ; }

			// v は、値なので、空の場合は、イベントが発生しない。
			if( "v".equals(qname) ) {		// v の時のみ、値出力を行う。
				// Process the last contents as required.
				// Do now, as characters() may be called more than once
				if(isSST) {
					int idx = Integer.parseInt(lastContents);
					lastContents = new XSSFRichTextString(sst.getEntryAt(idx)).toString();
					isSST = false;	// 実はこの必要性が判らない。
				}

				// v => contents of a cell
				// Output after we've seen the string contents
				//           文字列(値)    行      列
				event.value( lastContents, rowNo , colNo );
			}
		}

		/**
		 * 要素内の文字データの通知を受け取ります。
		 *
		 * インタフェース ContentHandler 内の characters メソッドをオーバーライドしています。
		 * 各文字データチャンクに対して特殊なアクション (ノードまたはバッファへのデータの追加、
		 * データのファイルへの出力など) を実行することができます。
		 *
		 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
		 *
		 * @param	buffer	文字データ配列
		 * @param	start	配列内の開始位置
		 * @param	length	配列から読み取られる文字数
		 * @see org.xml.sax.helpers.DefaultHandler#characters(char[] , int , int )
		 */
		@Override
		public void characters(char[] buffer, int start, int length) {
			if( isRowSkip ) { return ; }

			lastContents += new String( buffer, start, length );		// StringBuilder#append より速かった。
		}
	}

	/**
	 * シート一覧を、XSSFReader から取得します。
	 *
	 * 取得元が、XSSFReader なので、xlsx 形式のみの対応です。
	 * 汎用的なメソッドではなく、大きな xlsx ファイルは、通常の DOM処理すると、
	 * 大量のメモリを消費する為、イベントモデルで処理する場合に、使います。
	 *
	 * EXCEL上のシート名を、配列で返します。
	 *
	 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
	 *
	 * @param	rd XSSFReaderオブジェクト
	 * @return	シート名の配列
	 * @throws	SAXException			SAX の一般的なエラーが発生
	 * @throws	InvalidFormatException	よみとったEXCEL ファイルのフォーマットが異なる。
	 * @throws	IOException				SAXパース処理時のI/Oエラー
	 */
	public static String[] getSheetNames( final XSSFReader rd ) throws SAXException,IOException,InvalidFormatException {
		final List<String> shtName = new ArrayList<String>();

		XMLReader parser = XMLReaderFactory.createXMLReader( SAX_PARSER );		// SAXException
		parser.setContentHandler(
			new DefaultHandler() {
				public void startElement( final String uri, final String localName, final String name, final Attributes attributes) {
					if("sheet".equals(name)) {
						String shtNm = attributes.getValue("name");		// シート名
				//		String shtId = attributes.getValue("r:id");		// シートID( rId#  #は、１から始まる )
						shtName.add( shtNm );
					}
				}
			}
		);

		InputStream workbk = null;
		try {
			workbk = rd.getWorkbookData();										// IOException,InvalidFormatException
			InputSource wkbkSource = new InputSource( workbk );
			parser.parse(wkbkSource);											// IOException
		}
		finally {
			Closer.ioClose( workbk );
		}

		return shtName.toArray( new String[shtName.size()] );
	}

	/**
	 * アプリケーションのサンプルです。
	 *
	 * 入力ファイル名 は必須で、第一引数固定です。
	 *
	 * Usage: java org.opengion.fukurou.util.ExcelReader_XSSF 入力ファイル名
	 *
	 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
	 *
	 * @param	args	コマンド引数配列
	 */
	public static void main( final String[] args ) {
		final String usageMsg = "Usage: java org.opengion.fukurou.util.ExcelReader_XSSF 入力ファイル名" ;
		if( args.length == 0 ) {
			System.err.println( usageMsg );
			return ;
		}

		String fname = args[0];

		ExcelReader_XSSF.excelReader(
			fname,
			new ExcelReaderEvent() {
				public boolean startSheet( final String shtNm,final int shtNo ) {
					System.out.println( "S[" + shtNo + "]=" + shtNm );
					return super.startSheet( shtNm,shtNo );
				}

		//		public void columnNames( final String[] names ) {
		//			System.out.println( "NM=" + java.util.Arrays.toString( names ) );
		//		}

		//		public void values( final String[] vals,final int rowNo ) {
		//			System.out.println( "V[" + rowNo + "]=" + java.util.Arrays.toString( vals ) );
		//		}

		//		public boolean isSkip( final int rowNo ) {
		//			super.isSkip( rowNo );
		//			return false;
		//		}

				public boolean value( final String val,final int rowNo,final int colNo ) {
					System.out.println( "R[" + rowNo + "],C[" + colNo + "]=" + val );
					return super.value( val,rowNo,colNo );
				}
			}
		);
	}
}
