/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.plugin.io;

import java.io.BufferedReader;

import org.opengion.fukurou.model.ExcelModel;				// 6.0.2.0 (2014/09/19) 新規追加
// import org.opengion.fukurou.util.Closer;
import org.opengion.fukurou.util.StringUtil;

import org.opengion.hayabusa.common.HybsSystem;
import org.opengion.hayabusa.common.HybsSystemException;
import org.opengion.hayabusa.db.DBTableModelUtil;

/**
 * POI による、EXCELバイナリファイルを読み取る実装クラスです。
 *
 * ファイル名、シート名を指定して、データを読み取ることが可能です。
 * 第一カラムが # で始まる行は、コメント行なので、読み飛ばします。
 * カラム名の指定行で、カラム名が null の場合は、その列は読み飛ばします。
 *
 * 入力形式は、openXML形式にも対応しています。
 * ファイルの内容に応じて、.xlsと.xlsxのどちらで読み取るかは、内部的に
 * 自動判定されます。
 *
 * @og.rev 3.5.4.8 (2004/02/23) 新規作成
 * @og.rev 4.3.6.7 (2009/05/22) ooxml形式対応
 * @og.group ファイル入力
 *
 * @version  4.0
 * @author   Kazuhiko Hasegawa
 * @since    JDK5.0,
 */
public class TableReader_Excel extends TableReader_Default {
	//* このプログラムのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "6.0.2.5 (2014/10/31)" ;

	private String  filename		= null;		// 3.5.4.3 (2004/01/05)
	private String  sheetName		= null;		// 3.5.4.2 (2003/12/15)
	private String  sheetNos		= null;		// 5.5.7.2 (2012/10/09)

	private String  constKeys		= null;		// 5.5.8.2 (2012/11/09) 固定値となるカラム名(CSV形式)
	private String  constAdrs		= null;		// 5.5.8.2 (2012/11/09) 固定値となるアドレス(行-列,行-列,・・・)
	private String	nullBreakClm	= null;		// 5.5.8.2 (2012/11/09) 取込み条件/Sheet BREAK条件

//	private ExcelModel excel		= null;		// 6.0.2.0 (2014/09/19) 新規追加

	/**
	 * DBTableModel から 各形式のデータを作成して,BufferedReader より読み取ります。
	 * コメント/空行を除き、最初の行は、必ず項目名が必要です。
	 * それ以降は、コメント/空行を除き、データとして読み込んでいきます。
	 * このメソッドは、EXCEL 読み込み時に使用します。
	 *
	 * @og.rev 4.0.0.0 (2006/09/31) 新規追加
	 * @og.rev 5.1.6.0 (2010/05/01) columns 処理 追加
	 * @og.rev 5.1.6.0 (2010/05/01) skipRowCountの追加
	 * @og.rev 5.1.8.0 (2010/07/01) Exception をきちっと記述(InvalidFormatException)
	 * @og.rev 5.2.1.0 (2010/10/01) setTableColumnValues メソッドを経由して、テーブルにデータをセットする。
	 * @og.rev 5.5.1.2 (2012/04/06) HeaderData を try の上にだし、エラーメッセージを取得できるようにする。
	 * @og.rev 5.5.7.2 (2012/10/09) sheetNos 追加による複数シートのマージ読み取りサポート
	 * @og.rev 5.5.8.2 (2012/11/09) HeaderData に デバッグフラグを渡します。
	 * @og.rev 6.0.2.5 (2014/10/31) debug=true 時に、進捗が見えるようにします。
	 *
	 * @see #isExcel()
	 */
	@Override
	public void readDBTable() {
		HeaderData data = null;					// 5.5.1.2 (2012/04/06)
		boolean isDebug = isDebug();			// 5.5.7.2 (2012/10/09) デバッグ情報

		if( isDebug ) { System.out.println( " Filename=" + filename ) ; }

//		excel = new ExcelModel( filename,true );
		ExcelModel excel = new ExcelModel( filename,true );	// 6.0.2.5 (2014/10/31) refactoring:ローカル変数化

		int[] sheets = null;								// 6.0.2.0 (2014/09/19) Sheetオブジェクトではなく、シート番号の配列

	//	if( isDebug ) { excel.activeWorkbook( true ); }		// デバッグモード時には、エクセルのアクティブセル領域のみにシュリンクを行う

		// 5.5.7.2 (2012/10/09) 複数シートのマージ読み取り。 sheetNos の指定が優先される。
		if( sheetNos != null && sheetNos.length() > 0 ) {
			String[] sheetList = StringUtil.csv2ArrayExt( sheetNos , excel.getNumberOfSheets()-1 );	// 最大シート番号は、シート数-1
			sheets = new int[sheetList.length];
			for( int i=0; i<sheetList.length; i++ ) {
				sheets[i] = Integer.parseInt( sheetList[i] );
			}
		}
		else if( sheetName != null && sheetName.length() > 0 ) {
			int shNo = excel.getSheetNo( sheetName );
			if( shNo < 0 ) {
				String errMsg = "対応するシートが存在しません。 Sheet=[" + sheetName + "]" ;
				throw new HybsSystemException( errMsg );
			}
			sheets = new int[] { shNo };
		}
		else {
			sheets = new int[] { 0 };
		}

		boolean  nameNoSet = true;
		table = DBTableModelUtil.newDBTable();

		int numberOfRows = 0;
		data = new HeaderData();				// 5.5.1.2 (2012/04/06)

		data.setDebug( isDebug );				// 5.5.8.2 (2012/11/09)

		// 5.1.6.0 (2010/05/01) columns 処理
		data.setUseNumber( isUseNumber() );

		// 5.5.8.2 (2012/11/09) 固定値となるカラム名(CSV形式)とアドレス(行-列,行-列,・・・)を設定
		data.setSheetConstData( constKeys,constAdrs );

		int nullBreakClmAdrs = -1;						// 5.5.8.2 (2012/11/09) nullBreakClm の DBTableModel上のアドレス。-1 は、未使用
		if( data.setColumns( columns ) ) {
			nameNoSet = false;
			table.init( data.getColumnSize() );
			setTableDBColumn( data.getNames() ) ;
			nullBreakClmAdrs = table.getColumnNo( nullBreakClm, false );	// 5.5.8.2 (2012/11/09) カラム番号取得。存在しなければ -1 を返す。
		}

		int skip = getSkipRowCount();					// 5.1.6.0 (2010/05/01)
		String[] vals ;									// 6.0.2.5 (2014/10/31) ループから出す。
		String[] tblData ;								// 6.0.2.5 (2014/10/31) ループから出す。
		// 5.5.7.2 (2012/10/09) 複数シートのマージ読み取り。 
		for( int i=0; i<sheets.length; i++ ) {			// 5.5.7.2 (2012/10/09) シート配列を処理します。
//			int   shtNo = sheets[i] ;
//			String shtName = excel.getSheetName( shtNo );
			String shtName = excel.getSheetName( sheets[i] );	// 6.0.2.5 (2014/10/31) 中間変数を使用しない

			// 先に、getSheetName を実行して、内部の Sheet を確定させる必要がある。
			data.setSheetConstValues( excel,shtName );		// 6.0.2.0 (2014/09/19) Sheet ではなく、ExcelModel を渡します。

			int nFirstRow = excel.getFirstRowNum();
			if( nFirstRow < skip ) { nFirstRow = skip; }	// 5.1.6.0 (2010/05/01)
			int nLastRow  = excel.getLastRowNum();
			if( isDebug ) {									// 5.5.7.2 (2012/10/09) デバッグ情報
				System.out.println( " Debug: 行連番=" + numberOfRows + " : Sheet=" + shtName + " , 開始=" + nFirstRow + " , 終了=" + nLastRow );
			}
			for( int rowNo = nFirstRow; rowNo <= nLastRow; rowNo++) {
				// 6.0.2.5 (2014/10/31) debug=true 時に、進捗が見えるようにします。
				if( isDebug && rowNo > 0 ) {				// rowN0==0 の時は、必ず、余も 0 なので、出力しない。
					if( rowNo % 50   == 0 ) { System.err.print( "." ); }
					if( rowNo % 1000 == 0 ) { System.err.println( "  Count=[" + rowNo + "]" ); }
				}
//				String[] vals = excel.getValues(rowNo);
				vals = excel.getValues(rowNo);				// 6.0.2.5 (2014/10/31) ループから出す。
				if( data.isSkip( vals ) ) { continue; }		// 6.0.2.0 (2014/09/19) Row ではなく、行の文字列配列を渡します。
				if( nameNoSet ) {
					nameNoSet = false;
					table.init( data.getColumnSize() );
					setTableDBColumn( data.getNames() ) ;
					nullBreakClmAdrs = table.getColumnNo( nullBreakClm, false );	// 5.5.8.2 (2012/11/09) カラム番号取得。存在しなければ -1 を返す。
				}

				if( numberOfRows < getMaxRowCount() ) {
//					String[] tblData = data.row2Array( vals );	// 6.0.2.0 (2014/09/19) Row ではなく、行の文字列配列を渡します。
					tblData = data.row2Array( vals );			// 6.0.2.5 (2014/10/31) ループから出す。
					if( nullBreakClmAdrs >= 0 && ( tblData[nullBreakClmAdrs] == null || tblData[nullBreakClmAdrs].isEmpty() ) ) {
						break;		// nullBreakClm が null の場合は、そのSheet処理を中止する。
					}
					setTableColumnValues( tblData );					// 5.5.8.2 (2012/11/09)
					numberOfRows ++ ;
				}
				else {
					table.setOverflow( true );
				}
			}

			if( isDebug ) {				// 6.0.2.5 (2014/10/31) 最後は改行で終わる。
				System.err.println( "  Count=[" + nLastRow + "]" );
			}

			// 最後まで、#NAME が見つから無かった場合
			if( nameNoSet ) {
				String errMsg = "最後まで、#NAME が見つかりませんでした。"
								+ HybsSystem.CR
								+ "ファイルが空か、もしくは損傷している可能性があります。"
								+ HybsSystem.CR ;
				throw new HybsSystemException( errMsg );
			}
		}
	}

	/**
	 * DBTableModel から 各形式のデータを作成して,BufferedReader より読み取ります。
	 * コメント/空行を除き、最初の行は、必ず項目名が必要です。
	 * それ以降は、コメント/空行を除き、データとして読み込んでいきます。
	 *
	 * @og.rev 3.5.4.3 (2004/01/05) 引数に、BufferedReader を受け取る要に変更します。
	 * @og.rev 4.0.0.0 (2006/09/31) UnsupportedOperationException を発行します。
	 *
	 * @param   reader 各形式のデータ(使用していません)
	 */
	@Override
	public void readDBTable( final BufferedReader reader ) {
		String errMsg = "このクラスでは実装されていません。";
		throw new UnsupportedOperationException( errMsg );
	}

	/**
	 * DBTableModelのデータとしてEXCELファイルを読み込むときのシート名を設定します。
	 * これにより、複数の形式の異なるデータを順次読み込むことや、シートを指定して
	 * 読み取ることが可能になります。
	 * sheetNos と sheetName が同時に指定された場合は、sheetNos が優先されます。エラーにはならないのでご注意ください。
	 * のでご注意ください。
	 *
	 * @og.rev 3.5.4.2 (2003/12/15) 新規追加
	 *
	 * @param   sheetName シート名
	 */
	@Override
	public void setSheetName( final String sheetName ) {
		this.sheetName = sheetName;
	}

	/**
	 * EXCELファイルを読み込むときのシート番号を指定します(初期値:0)。
	 *
	 * EXCEL読み込み時に複数シートをマージして取り込みます。
	 * シート番号は、0 から始まる数字で表します。
	 * ヘッダーは、最初のシートのカラム位置に合わせます。（ヘッダータイトルの自動認識はありません。）
	 * よって、指定するシートは、すべて同一レイアウトでないと取り込み時にカラムのずれが発生します。
	 * 
	 * シート番号の指定は、カンマ区切りで、複数指定できます。また、N-M の様にハイフンで繋げることで、
	 * N 番から、M 番のシート範囲を一括指定可能です。また、"*" による、全シート指定が可能です。
	 * これらの組み合わせも可能です。（ 0,1,3,5-8,10-* ）
	 * ただし、"*" に関しては例外的に、一文字だけで、すべてのシートを表すか、N-* を最後に指定するかの
	 * どちらかです。途中には、"*" は、現れません。
	 * シート番号は、重複(1,1,2,2)、逆転(3,2,1) での指定が可能です。これは、その指定順で、読み込まれます。
	 * sheetNos と sheetName が同時に指定された場合は、sheetNos が優先されます。エラーにはならないのでご注意ください。
	 * このメソッドは、isExcel() == true の場合のみ利用されます。
	 * 
	 * 初期値は、0（第一シート） です。
	 *
	 * ※ このクラスでは実装されていません。
	 *
	 * @og.rev 5.5.7.2 (2012/10/09) 新規追加
	 *
	 * @param   sheetNos EXCELファイルのシート番号（0から始まる）
	 * @see		#setSheetName( String ) 
	 */
	@Override
	public void setSheetNos( final String sheetNos ) {
		this.sheetNos = sheetNos;
	}

	/**
	 * EXCELファイルを読み込むときのシート単位の固定値を設定するためのカラム名とアドレスを指定します。
	 * カラム名は、カンマ区切りで指定します。
	 * 対応するアドレスを、EXCEL上の行-列を０から始まる整数でカンマ区切りで指定します。
	 * これにより、シートの一か所に書かれている情報を、DBTableModel のカラムに固定値として
	 * 設定することができます。
	 * 例として、DB定義書で、テーブル名をシートの全レコードに設定したい場合などに使います。
	 * このメソッドは、isExcel() == true の場合のみ利用されます。
	 *
	 * @og.rev 5.5.8.2 (2012/11/09) 新規追加
	 *
	 * @param   constKeys 固定値となるカラム名(CSV形式)
	 * @param   constAdrs 固定値となるアドレス(行-列,行-列,・・・)
	 */
	@Override
	public void setSheetConstData( final String constKeys,final String constAdrs ) {
		this.constKeys = constKeys;
		this.constAdrs = constAdrs;
	}

	/**
	 * ここに指定されたカラム列に NULL が現れた時点で読み取りを中止します。
	 *
	 * これは、指定のカラムは必須という事を条件に、そのレコードだけを読み取る処理を行います。
	 * 複数Sheetの場合は、次のSheetを読みます。
	 * 現時点では、Excel の場合のみ有効です。
	 *
	 * @og.rev 5.5.8.2 (2012/11/09) 新規追加
	 *
	 * @param   clm カラム列
	 */
	@Override
	public void setNullBreakClm( final String clm ) {
		nullBreakClm = clm;
	}

	/**
	 * このクラスが、EXCEL対応機能を持っているかどうかを返します。
	 *
	 * EXCEL対応機能とは、シート名のセット、読み込み元ファイルの
	 * Fileオブジェクト取得などの、特殊機能です。
	 * 本来は、インターフェースを分けるべきと考えますが、taglib クラス等の
	 * 関係があり、問い合わせによる条件分岐で対応します。
	 *
	 * @og.rev 3.5.4.3 (2004/01/05) 新規追加
	 *
	 * @return	EXCEL対応機能を持っているかどうか(常にtrue)
	 */
	@Override
	public boolean isExcel() {
		return true;
	}

	/**
	 * 読み取り元ファイル名をセットします。(DIR + Filename)
	 * これは、EXCEL追加機能として実装されています。
	 *
	 * @og.rev 3.5.4.3 (2004/01/05) 新規作成
	 *
	 * @param   filename 読み取り元ファイル名
	 */
	@Override
	public void setFilename( final String filename ) {
		this.filename = filename;
		if( filename == null ) {
			String errMsg = "ファイル名が指定されていません。" ;
			throw new HybsSystemException( errMsg );
		}
	}
}

/**
 * EXCEL ネイティブのデータを処理する ローカルクラスです。
 * このクラスでは、コメント行のスキップ判定、ヘッダー部のカラム名取得、
 * 行情報(Row)から、カラムの配列の取得などを行います。
 *
 * @og.rev 3.5.4.8 (2004/02/23) 新規追加
 * @og.group ファイル入力
 *
 * @version  4.0
 * @author   儲
 * @since    JDK5.0,
 */
class HeaderData {
	private String[] names ;
	private int[]    index;					// 4.3.4.0 (2008/12/01) POI3.2対応
	private int		 columnSize = 0;
	private boolean  nameNoSet = true;
	private boolean  useNumber = true;
	private boolean  isDebug   = false;		// 5.5.8.2 (2012/11/09)

//	private String[] orgNames ;				// 5.5.1.2 (2012/04/06) オリジナルのカラム名	// 6.0.2.5 (2014/10/31) refactoring

	// 5.5.8.2 (2012/11/09) 固定値のカラム名、DBTableModelのアドレス、Sheetの行-列番号
	private int		 cnstLen = 0;			// 初期値=0 の場合は、固定値を使わないという事。
	private String[] cnstKeys ;
	private int[]	 cnstIndx ;
	private int[]    cnstRowNo;
	private int[]    cnstClmNo;
	private String[] cnstVals ;				// Sheet単位の固定値のキャッシュ(シートの最初に値を取得して保持しておく)

	/**
	 * デバッグ情報を、出力するかどうか[true/false]を指定します(初期値:false)。
	 *
	 * 初期値は、false(出力しない) です。
	 *
	 * @og.rev 5.5.8.2 (2012/11/09) 新規作成
	 *
	 * @param	isDebug	デバッグ情報 [true:出力する/false:出力しない]
	 */
	void setDebug( final boolean isDebug ) {
		this.isDebug = isDebug ;
	}

	/**
	 * 行番号情報を、使用しているかどうか[true/false]を指定します(初期値:true)。
	 *
	 * 初期値は、true(使用する) です。
	 *
	 * @og.rev 5.1.6.0 (2010/05/01) 新規作成
	 *
	 * @param	useNumber	行番号情報 [true:使用している/false:していない]
	 */
	void setUseNumber( final boolean useNumber ) {
		this.useNumber = useNumber ;
	}

	/**
	 * 固定値となるカラム名(CSV形式)と、constAdrs 固定値となるアドレス(行-列,行-列,・・・)を設定します。
	 *
	 * アドレスは、EXCEL上の行-列をカンマ区切りで指定します。
	 * 行列は、EXCELオブジェクトに準拠するため、０から始まる整数です。
	 * 0-0 ⇒ A1 , 1-0 ⇒ A2 , 0-1 ⇒ B1 になります。
	 * これにより、シートの一か所に書かれている情報を、DBTableModel のカラムに固定値として
	 * 設定することができます。
	 * 例として、DB定義書で、テーブル名をシートの全レコードに設定したい場合などに使います。
	 * このメソッドは、isExcel() == true の場合のみ利用されます。
	 *
	 * 5.7.6.3 (2014/05/23) より、
	 *   ①EXCEL表記に準拠した、A1,A2,B1 の記述も処理できるように対応します。
	 *     なお、A1,A2,B1 の記述は、必ず、英字1文字＋数字 にしてください。(A～Zまで)
	 *   ②処理中のEXCELシート名をカラムに割り当てるために、"SHEET" という記号に対応します。
	 * 例えば、sheetConstKeys="CLM,LANG,NAME" とし、sheetConstAdrs="0-0,A2,SHEET" とすると、
	 * NAMEカラムには、シート名を読み込むことができます。
	 * これは、内部処理の簡素化のためです。
	 *
	 * ちなみに、EXCELのセルに、シート名を表示させる場合の関数は、下記の様になります。
	 * =RIGHT(CELL("filename",$A$1),LEN(CELL("filename",$A$1))-FIND("]",CELL("filename",$A$1)))
	 *
	 * @param	constKeys	固定値となるカラム名(CSV形式)
	 * @param	constAdrs	固定値となるアドレス(行-列,行-列,・・・)
	 *
	 * @og.rev 5.5.8.2 (2012/11/09) 新規追加
	 * @og.rev 5.7.6.3 (2014/05/23) EXCEL表記(A2,B1等)の対応と、特殊記号(SHEET)の対応
	 */
	void setSheetConstData( final String constKeys,final String constAdrs ) {
		if( constKeys == null || constKeys.isEmpty() ) {
			return ;
		}

		cnstKeys  = constKeys.split( "," );
		cnstLen   = cnstKeys.length;
		cnstIndx  = new int[cnstLen];
		cnstRowNo = new int[cnstLen];
		cnstClmNo = new int[cnstLen];

		String[] row_col = constAdrs.split( "," ) ;
		cnstRowNo = new int[cnstLen];
		cnstClmNo = new int[cnstLen];
		for( int j=0; j<cnstLen; j++ ) {
			cnstKeys[j] = cnstKeys[j].trim();		// 前後の不要なスペースを削除
			String rowcol = row_col[j].trim();		// 前後の不要なスペースを削除

			// 5.7.6.3 (2014/05/23) EXCEL表記(A2,B1等)の対応と、特殊記号(SHEET)の対応
			int sep = rowcol.indexOf( '-' );
			if( sep > 0 ) {
				cnstRowNo[j] = Integer.parseInt( rowcol.substring( 0,sep ) );
				cnstClmNo[j] = Integer.parseInt( rowcol.substring( sep+1 ) );
			}
			else {
				if( "SHEET".equalsIgnoreCase( rowcol ) ) {		// "SHEET" 時は、cnstRowNo をマイナスにしておきます。
					cnstRowNo[j] = -1 ;
					cnstClmNo[j] = -1 ;
				}
				else if( rowcol.length() >= 2 ) {
					cnstRowNo[j] = Integer.parseInt( rowcol.substring( 1 ) ) -1;	// C6 の場合、RowNoは、6-1=5
					cnstClmNo[j] = rowcol.charAt(0) - 'A' ;							// C6 の場合、'C'-'A'=2
				}
			}

			if( isDebug ) {
				System.out.println( " Debug: constKey=" + cnstKeys[j] + " : RowNo=" + cnstRowNo[j] + " , ClmNo=" + cnstClmNo[j] );
			}
		}
	}

	/**
	 * カラム名を外部から指定します。
	 * カラム名が、NULL でなければ、＃NAME より、こちらが優先されます。
	 * カラム名は、順番に、指定する必要があります。
	 *
	 * @og.rev 5.1.6.0 (2010/05/01) 新規作成
	 * @og.rev 5.5.8.2 (2012/11/09) 固定値取得用の cnstIndx の設定を行う。
	 *
	 * @param	columns	EXCELのカラム列(CSV形式)
	 *
	 * @return true:処理実施/false:無処理
	 */
	boolean setColumns( final String columns ) {
		if( columns != null && columns.length() > 0 ) {
			names = StringUtil.csv2Array( columns );
			columnSize = names.length ;
			index = new int[columnSize];
			int adrs = useNumber ? 1:0 ;	// useNumber =true の場合は、１件目(No)は読み飛ばす。
			// 5.5.8.2 (2012/11/09) 固定値取得用の cnstIndx の設定を行う。
			for( int i=0; i<columnSize; i++ ) {
				index[i] = adrs++;
				for( int j=0; j<cnstLen; j++ ) {
					if( names[i].equalsIgnoreCase( cnstKeys[j] ) ) {
						cnstIndx[j] = index[i];
					}
				}
			}
			nameNoSet = false;

			return true;
		}
		return false;
	}

	/**
	 * EXCEL ネイティブのデータを処理する ローカルクラスです。
	 * このクラスでは、コメント行のスキップ判定、ヘッダー部のカラム名取得、
	 * 行情報(Row)から、カラムの配列の取得などを行います。
	 *
	 * @og.rev 4.3.4.0 (2008/12/01) POI3.2対応
	 * @og.rev 6.0.2.0 (2014/09/19) Row ではなく、行の文字列配列を渡します。
	 *
	 * @param vals EXCELの行から取得した文字列配列
	 *
	 * @return true:コメント行/false:通常行
	 */
	boolean isSkip( final String[] vals ) {
		if( vals == null || vals.length == 0 ) { return true; }

		String strText =  vals[0];
		if( strText != null && strText.length() > 0 ) {
			if( nameNoSet ) {
				if( "#Name".equalsIgnoreCase( strText ) ) {
					makeNames( vals );							// 6.0.2.0 (2014/09/19) Row ではなく、行の文字列配列を渡します。
					nameNoSet = false;
					return true;
				}
				else if( strText.charAt( 0 ) == '#' ) {
					return true;
				}
				else {
					String errMsg = "#NAME が見つかる前にデータが見つかりました。"
									+ HybsSystem.CR
									+ "可能性として、ファイルが、ネイティブExcelでない事が考えられます。"
									+ HybsSystem.CR ;
					throw new HybsSystemException( errMsg );
				}
			}
			else {
				if( strText.charAt( 0 ) == '#' ) {
					return true;
				}
			}
		}

		return nameNoSet ;
	}

	/**
	 * EXCELの行から取得した文字列配列からカラム名情報を取得します。
	 *
	 * @og.rev 4.3.4.0 (2008/12/01) POI3.2対応
	 * @og.rev 5.1.6.0 (2010/05/01) useNumber(行番号情報を、使用している(true)/していない(false)を指定)
	 * @og.rev 5.1.6.0 (2010/05/01) useNumber(行番号情報を、使用している(true)/していない(false)を指定)
	 * @og.rev 5.5.1.2 (2012/04/06) オリジナルのカラム名を取得
	 * @og.rev 5.5.8.2 (2012/11/09) 固定値取得用の cnstIndx の設定を行う。
	 * @og.rev 6.0.2.0 (2014/09/19) Row ではなく、行の文字列配列を渡します。
	 *
	 * @param vals EXCELの行から取得した文字列配列
	 */
	private void makeNames( final String[] vals ) {
		// 先頭カラムは、#NAME 属性行であるかどうかを、useNumber で判定しておく。
		short nFirstCell = (short)( useNumber ? 1:0 );

//		orgNames = vals;		// 6.0.2.5 (2014/10/31) refactoring 読み出されないフィールド:orgNames

		int maxCnt = vals.length - nFirstCell ;
		String[] names2 = new String[maxCnt];
		int[]    index2 = new int[maxCnt];

		// 先頭カラムは、#NAME 属性行である。++ で、一つ進めている。
		// 先頭カラムは、#NAME 属性行であるかどうかを、useNumber で判定しておく。
		for( int nIndexCell = nFirstCell; nIndexCell < vals.length; nIndexCell++) {
			String strText = vals[nIndexCell];

			// #NAME 行が、ゼロ文字列の場合は、読み飛ばす。
			if( strText != null && strText.length() > 0 ) {
				names2[columnSize] = strText;
				index2[columnSize] = nIndexCell;
				columnSize++;
			}
		}

		// #NAME を使用しない場合：no欄が存在しないケース
		if( maxCnt == columnSize ) {
			names = names2;
			index = index2;
		}
		else {
			names = new String[columnSize];
			index = new int[columnSize];
			System.arraycopy(names2, 0, names, 0, columnSize);
			System.arraycopy(index2, 0, index, 0, columnSize);
		}

		// 5.5.8.2 (2012/11/09) 固定値取得用の cnstIndx の設定を行う。
		if( cnstLen > 0 ) {
			for( int i=0; i<columnSize; i++ ) {
				for( int j=0; j<cnstLen; j++ ) {
					if( names[i].equalsIgnoreCase( cnstKeys[j] ) ) {
						cnstIndx[j] = index[i];
					}
				}
			}
		}
	}

	/**
	 * カラム名情報を返します。
	 * ここでは、内部配列をそのまま返します。
	 *
	 * @return String[] カラム列配列情報
	 */
	String[] getNames() {
		return names;
	}

	/**
	 * カラムサイズを返します。
	 *
	 * @return	カラムサイズ
	 */
	int getColumnSize() {
		return columnSize;
	}

	/**
	 * Sheet単位の固定値のキャッシュ(シートの最初に値を取得して保持しておく)を設定します。
	 * これは、シートチェンジの最初に一度呼び出しておくことで、それ以降の列取得時に
	 * 固定値を利用することで処理速度向上を目指します。
	 *
	 * "SHEET" が指定された場合は、cnstRowNo[j]=-1 が設定されている。
	 *
	 * @og.rev 5.5.8.2 (2012/11/09) 新規作成
	 * @og.rev 5.7.6.3 (2014/05/23) 特殊記号(SHEET)の対応
	 *
	 * @param excel		ExcelModelオブジェクト
	 * @param shtName	Sheet名
	 */
	void setSheetConstValues( final ExcelModel excel,final String shtName ) {
		cnstVals = new String[cnstLen];
		for( int j=0; j<cnstLen; j++ ) {
			// 5.7.6.3 (2014/05/23) 特殊記号(SHEET)の対応
			if( cnstRowNo[j] < 0 ) {
				cnstVals[j] = shtName ;
			}
			else {
				cnstVals[j] = excel.getValue( cnstRowNo[j],cnstClmNo[j] );
			}

			if( isDebug ) {
				System.out.println( " Debug: Sheet=" + shtName + " : RowNo=" + cnstRowNo[j] + " , ClmNo=" + cnstClmNo[j] + " , " + cnstKeys[j] + "=" + cnstVals[j] );
			}
		}
	}

	/**
	 * カラム名情報を返します。
	 *
	 * @og.rev 5.5.8.2 (2012/11/09) 固定値の設定を行う。
	 * @og.rev 6.0.2.0 (2014/09/19) Row ではなく、行の文字列配列を渡します。
	 *
	 * @param  vals		行の文字列配列
	 * @return String[] カラム列配列情報
	 */
	String[] row2Array( final String[] vals ) {
		if( nameNoSet ) {
			String errMsg = "#NAME が見つかる前にデータが見つかりました。";
			throw new HybsSystemException( errMsg );
		}

		String[] data = new String[columnSize];
		for( int i=0;i<columnSize; i++ ) {
			data[i] = vals[index[i]];
		}

		// 5.5.8.2 (2012/11/09) 固定値の設定を行う。
		for( int j=0; j<cnstLen; j++ ) {
			data[cnstIndx[j]] = cnstVals[j];
		}
		return data;
	}
}
